/* 
 * chasen.h - header file for ChaSen library
 *
 * $Id: chasen.h,v 1.3 2001/02/24 06:17:22 kazuma-t Exp $
 */

#ifndef __CHASEN_H__
#define __CHASEN_H__

/* variables */
extern int Cha_optind;

/* functions */
extern int chasen_getopt_argv(char**, FILE*);
extern int chasen_fparse(FILE*, FILE*);
extern int chasen_sparse(char*, FILE*);
extern char *chasen_fparse_tostr(FILE*);
extern char *chasen_sparse_tostr(char*);

#endif /* __CHASEN_H__ */
